@@ -21,6 +21,12 @@ Setting up a Newsletter, and subscribing people to it.
     >>> n.subscribers()
     [{'secret': '...', 'email': 'bla@example.com', 'subscribed': True}]
 
+    >>> n.subscribe("notbla@example.com", subscribed = True)
+    True
+
+    >>> len(n.subscribers())
+    2
+
 
 
 Setting up a NewletterSend and sending email to people on it.
@@ -40,7 +46,7 @@ Check that the users have been added correctly.
 
     >>> import pprint
     >>> len(list(ns._send_parts.items()))
-    1
+    2
 
     >>> pprint.pprint(list(sorted([sp for sp in ns._send_parts.values()])))
     [{'body': 'this is the body of the message',
@@ -50,9 +56,24 @@ Check that the users have been added correctly.
       'newsletter_send_id': '...',
       'status': 'open',
       'subject': 'a subject',
+      'user_id': '...'},
+     {'body': 'this is the body of the message',
+      'email': 'notbla@example.com',
+      'full_name': '',
+      'id': '...',
+      'newsletter_send_id': '...',
+      'status': 'open',
+      'subject': 'a subject',
       'user_id': '...'}]
 
 
+Make sure we can not add the same users multiple times to the same newsletter send.
+    >>> ns.add_users(newsletter_send_id,  n.subscribers())
+    True
+    >>> len(list(ns._send_parts.items()))
+    2
+
+
 Send the newsletter.
     >>> ns.send(newsletter_send_id)
     sending log {'body': 'this is the body of the message',
@@ -63,12 +84,18 @@ Send the newsletter.
      'status': 'sending',
      'subject': 'a subject',
      'user_id': '...'}
+    sending log {'body': 'this is the body of the message',
+     'email': 'notbla@example.com',
+     'full_name': '',
+     'id': '...',
+     'newsletter_send_id': '...',
+     'status': 'sending',
+     'subject': 'a subject',
+     'user_id': '...'}
     True
 
 
 
-
-
 Mini todo list.
 
 * actions
@@ -182,13 +209,19 @@ class Newsletter(object):
 
 
 
-    def subscribe(self, email, info = {}):
+    def subscribe(self, email, info = {}, subscribed = False):
         """
+            subscribed - if True we confirm the subscription automatically.  
+                         If False we ask the user to confirm the subscription.
         """
+        if subscribed not in [True, False]:
+            raise ValueError("subscribed needs to be True or False")
+
         user = {"email": email}
         user.update(info)
         user['secret'] = self._make_secret()
-        user['subscribed'] = False
+        user['subscribed'] = subscribed
+
 
         u = [u for u in self._subscribers.values() if u['email'] == email]
         if u:
@@ -198,7 +231,12 @@ class Newsletter(object):
             uid = self._make_uid()
             user["user_id"] = uid
             self._subscribers.update({uid: user})
-            self.send_subscribe_confirm_email(uid, user)
+            if subscribed:
+                # we confirm the subscription automatically.
+                self.confirm_subscribe(email = user['email'], secret = user['secret'])
+            else:
+                self.send_subscribe_confirm_email(uid, user)
+
 
         return True
 
@@ -542,12 +580,22 @@ class NewsletterSend(object):
         if ns['status'] != 'open':
             raise ValueError("Tried to add users to a send which is not open to send.")
 
-        for user in users:
+        # check that the same email is not already in there already.
+        users_not_in = self._get_users_not_in(users, self._send_parts)
+
+        for user in users_not_in:
             send_part = self._create_send_part(ns, user)
             self._send_parts.update({send_part['id']: send_part})
         
         return True
 
+    def _get_users_not_in(self, users, send_parts):
+        """ returns the users that are not already in the send_parts.  
+                Based on email address only.
+        """
+        send_part_emails = set([s['email'] for s in send_parts.values()])
+        return [u for u in users if u['email'] not in send_part_emails]
+
     def remove_user(self, newsletter_send_id, send_part_id):
         ns = self._get_newsletter_send(newsletter_send_id)
         
