@@ -265,7 +265,11 @@ class MyNewsletterSend(NewsletterSend):
         body = send_part['body']
 
         # hack to get the secret for the unsubscribe link.
-        secret = self._n._subscribers.get(send_part['user_id'])['secret']
+        the_user = self._n._subscribers.get(send_part['user_id'], None)
+        if the_user is None:
+            secret = self._n._subscribers.get(send_part['user_id'])['secret']
+        else:
+            secret = ""
 
         # replace template values.
         replace_vars = {"FULL_NAME" : send_part.get('full_name', ''),
@@ -383,6 +387,102 @@ class MyNewsletter(Newsletter):
 
 
 
+def create_SendMessages(mount_on, config):
+    """ from the config given search for send_message_ ones.
+        create instances of SendMessage on the object mount_on
+    """
+    for k in config.keys():
+        if k.startswith("send_message_"):
+            send_message_config = config[k]
+            if send_message_config['send_message']:
+                setattr(mount_on, 
+                        send_message_config["mount_point"], 
+                        SendMessage(config["newsletter"], 
+                                    send_message_config, 
+                                    mount_on))
+
+
+
+class SendMessage(object):
+    
+    def __init__(self, config, send_message_config, web_newsletter):
+        self.config = config
+        self.send_message_config = send_message_config
+        self.web_newsletter = web_newsletter
+
+    def __call__(self, **kwargs):
+        return self.send_message(**kwargs)
+    
+    def send_message(self, **kwargs):
+        """ send_message(**kwargs) sends message 
+
+            NOTE: Not exposed by default.  Need to enable it in config
+
+            kwargs are used as variables to replace in the body and subject.
+
+            wrap_it - if True we wrap html response in a header footer.
+            toaddr - a single email address to send to.
+        """
+        if not self.send_message_config['exposed']:
+            raise ValueError("not exposed")
+
+        if not self.validate_input(kwargs):
+            raise ValueError("not what we were expecting")
+        
+        wrap_it = kwargs.get('wrap_it', None)
+
+
+        email_template = self.send_message_config.get('email_template')
+        email_subject_template = self.send_message_config.get('email_subject_template')
+        template = self.send_message_config.get('template')
+        
+        toaddr = self.send_message_config.get('toaddr', '')
+
+        # We get the toaddr from the kwargs if it is set to None in the header
+        if toaddr is None:
+            toaddr = kwargs.get('toaddr')
+            if not validate_email(toaddr):
+                raise ValueError("toaddr invalid")
+        
+        if toaddr == '':
+            raise ValueError("toaddr invalid")
+
+        replace_vars = dict(kwargs)
+        replace_vars['toaddr'] = toaddr
+
+        subject = self.web_newsletter.admin.email_template(email_subject_template)
+        body = self.web_newsletter.admin.email_template(email_template)
+        
+
+        # replace the variables in the template.
+        subject = template_replace(subject, replace_vars)
+        body = template_replace(body, replace_vars)
+        
+        sendmail(self.send_message_config.get('fromaddr'), 
+                toaddr, subject, body, 
+                self.config.get('smtp_host'), 
+                self.config.get('smtp_username'), 
+                self.config.get('smtp_password'),
+                self.config.get('sendmail_cmd'),
+                )
+        
+        return self.web_newsletter.html_template(template, wrap_it)
+        
+        
+    def validate_input(self, kwargs):
+        """ returns True if the input is valid.
+        """
+        # see if a validation function has been used defined in the config file.
+        validate_input = self.send_message_config.get('validate_input', None)
+        if validate_input is not None:
+            return validate_input(kwargs)
+        
+        return True
+
+
+
+
+
 
 
 
@@ -395,12 +495,14 @@ class WebNewsletter(object):
         """ load(config) loads the application given the configuration.
         """
         self.config = config["newsletter"]
+
         self.static_dir = self.config.get("static_dir")
         self.db_dir = self.config.get("db_dir")
         self.fromaddr = self.config.get("from_addr")
 
         self._default_vars = {"base_url" : self.config.get('base_url'),
-                            }
+                             }
+
 
         self._n = MyNewsletter(self.config)
         self._ns = MyNewsletterSend(self.config)
@@ -411,6 +513,11 @@ class WebNewsletter(object):
 
         self.admin = WebNewsletterAdmin(self._n, self._ns, self.config)
 
+        # mount any send_message_ from the config.
+        create_SendMessages(self, config)
+
+
+
     def html_template(self, template_name, 
                             wrap_it = None, 
                             replace_vars = None):
@@ -422,6 +529,12 @@ class WebNewsletter(object):
         tmpl = os.path.join(self.static_dir, "web_front_templates", 
                             template_name + ".html")
         the_html = open(tmpl, "rb").read()
+
+        # get rid of trailing new lines
+        if the_html[-1] == "\n":
+            the_html = the_html[:-1]
+
+
         if replace_vars is None:
             replace_vars = {}
 
@@ -484,6 +597,10 @@ class WebNewsletter(object):
         if self._n.confirm_unsubscribe(email, secret):
             return self.html_template("confirm_unsubscribe", wrap_it)
 
+
+
+
+
     index.exposed = True
     subscribe.exposed = True
     confirm_subscribe.exposed = True
@@ -665,7 +782,13 @@ class WebNewsletterAdmin(object):
         tmpl = os.path.join(self.static_dir, 
                             "email_templates", 
                             template_name + ".html")
-        return open(tmpl, "rb").read()
+        the_html = open(tmpl, "rb").read()
+
+        # get rid of trailing new lines
+        if the_html[-1] == "\n":
+            the_html = the_html[:-1]
+
+        return the_html
 
     def edit(self, newsletter_send_id = None):
         """ edit(newsletter_send_id) edits the given newsletter send.
